#include "adc.h"

int tb_totalDevices, tbStatus;
unsigned int adcValue;
unsigned char rxBuffer[RX_BUFFER_SIZE]; /* This has to be unsigned for the data's sake */
unsigned char ROM_NO[8];

unsigned char	crc8;
int		LastDiscrepancy;
int 	LastFamilyDiscrepancy;
int 	LastDeviceFlag;

tinyboard* tinyboard_connect()
{
	tinyboard  *tempHandle = NULL;

	usb_init();
	usbOpenDevice(&tempHandle, VENDOR_ID, "*", PRODUCT_ID, "*", "*", NULL, NULL );

	return tempHandle;
}

void analog_init(tinyboard* tbHandle, unsigned char voltageRef)
{
	tbStatus=usb_control_msg(tbHandle, 0xC0, 35, (voltageRef<<8) | 0x07, 0, rxBuffer, 8, USB_TIMEOUT);
}

unsigned int analogRead(tinyboard* tbHandle, unsigned char channel)
{
	tbStatus=usb_control_msg(tbHandle, 0xC0, 15, channel, 0, rxBuffer, 8, USB_TIMEOUT);

	return ((rxBuffer[1] *256) + (rxBuffer[0]));
}

int usbGetStringAscii(usb_dev_handle *dev, int index, char *buf, int buflen)
{
char    buffer[256];
int     rval, i;

    if((rval = usb_get_string_simple(dev, index, buf, buflen)) >= 0) /* use libusb version if it works */
        return rval;
    if((rval = usb_control_msg(dev, USB_ENDPOINT_IN, USB_REQ_GET_DESCRIPTOR, (USB_DT_STRING << 8) + index, 0x0409, buffer, sizeof(buffer), 5000)) < 0)
        return rval;
    if(buffer[1] != USB_DT_STRING){
        *buf = 0;
        return 0;
    }
    if((unsigned char)buffer[0] < rval)
        rval = (unsigned char)buffer[0];
    rval /= 2;
    /* lossy conversion to ISO Latin1: */
    for(i=1;i<rval;i++){
        if(i > buflen)              /* destination buffer overflow */
            break;
        buf[i-1] = buffer[2 * i];
        if(buffer[2 * i + 1] != 0)  /* outside of ISO Latin1 range */
            buf[i-1] = '?';
    }
    buf[i-1] = 0;
    return i-1;
}

int usbOpenDevice(usb_dev_handle **device, int vendorID, char *vendorNamePattern, int productID, char *productNamePattern, char *serialNamePattern, FILE *printMatchingDevicesFp, FILE *warningsFp)
{
struct usb_bus      *bus;
struct usb_device   *dev;
usb_dev_handle      *handle = NULL;
int                 errorCode = USBOPEN_ERR_NOTFOUND;

    usb_find_busses();
    usb_find_devices();
    for(bus = usb_get_busses(); bus; bus = bus->next){
        for(dev = bus->devices; dev; dev = dev->next){  /* iterate over all devices on all busses */
            if((vendorID == 0 || dev->descriptor.idVendor == vendorID)
                        && (productID == 0 || dev->descriptor.idProduct == productID)){
                char    vendor[256], product[256], serial[256];
                int     len;
                handle = usb_open(dev); /* we need to open the device in order to query strings */
                if(!handle){
                    errorCode = USBOPEN_ERR_ACCESS;
                    if(warningsFp != NULL)
                        fprintf(warningsFp, "Warning: cannot open VID=0x%04x PID=0x%04x: %s\n", dev->descriptor.idVendor, dev->descriptor.idProduct, usb_strerror());
                    continue;
                }
                /* now check whether the names match: */
                len = vendor[0] = 0;
                if(dev->descriptor.iManufacturer > 0){
                    len = usbGetStringAscii(handle, dev->descriptor.iManufacturer, vendor, sizeof(vendor));
                }
                if(len < 0){
                    errorCode = USBOPEN_ERR_ACCESS;
                    if(warningsFp != NULL)
                        fprintf(warningsFp, "Warning: cannot query manufacturer for VID=0x%04x PID=0x%04x: %s\n", dev->descriptor.idVendor, dev->descriptor.idProduct, usb_strerror());
                }else{
                    errorCode = USBOPEN_ERR_NOTFOUND;
                    break;

                }
                usb_close(handle);
                handle = NULL;
            }
        }
        if(handle)  /* we have found a deice */
            break;
    }
    if(handle != NULL){
        errorCode = 0;
        *device = handle;
    }
    if(printMatchingDevicesFp != NULL)  /* never return an error for listing only */
        errorCode = 0;
    return errorCode;
}

void delay(unsigned int duration)
{
	#ifdef LINUX
		usleep(duration*1000);
	#else
		Sleep(duration);
	#endif
}

int main(int argc, char **argv)
{
	tinyboard *tb = NULL;

	tb = tinyboard_connect();

	if(tb == NULL){
		printf("> Micronucleus could not be found!\n");
		exit(EXIT_FAILURE);
	}

	// -- Select ADC reference voltage
	// Those definetly needs calibration for precision measurement.
	analog_init(tb,VREF_VCC);
	// analog_init(tb,VREF_1100mV);
	// analog_init(tb,VREF_2560mV;

	while(1)
	{
		adcValue=analogRead(tb, ADC_PIN2);

		printf("> Value: %d\tVoltage: %f milivolts\n", adcValue,(float)((adcValue*(3000))/1024.0));
		// printf("> Value: %d\tVoltage: %f milivolts\n", adcValue,(float)((adcValue*(1100))/1024.0));
		// printf("> Value: %d\tVoltage: %f milivolts\n", adcValue,(float)((adcValue*(2560))/1024.0));

		if(tbStatus<0)
		{
			printf("> tbStatus: %d\n",tbStatus);
			printf("> Connection error!\n");
			return 0;
		}

		delay(100);
	}
}
